/**
 **	$Header: /import/dev-vis/image/imtools/v2.0/imtools/src/RCS/imfeedback.raw.c,v 1.2 92/08/31 17:05:15 vle Exp $
 **	Copyright (c) 1989-1992  San Diego Supercomputer Center (SDSC)
 **		San Diego, California, USA
 **
 **	Users and possessors of this source code are hereby granted a
 **	nonexclusive, royalty-free copyright and design patent license to
 **	use this code in individual software.  License is not granted for
 **	commercial resale, in whole or in part, without prior written
 **	permission from SDSC.  This source is provided "AS IS" without express
 **	or implied warranty of any kind.
 **
 **	For further information contact:
 **		E-Mail:		info@sds.sdsc.edu
 **
 **		Surface Mail:	Information Center
 **				San Diego Supercomputer Center
 **				P.O. Box 85608
 **				San Diego, CA  92138-5608
 **				(619) 534-5000
 **/

#define HEADER	"    $Header: /import/dev-vis/image/imtools/v2.0/imtools/src/RCS/imfeedback.raw.c,v 1.2 92/08/31 17:05:15 vle Exp $"

/**
 **  FILE
 **	imfeedback.c	-  Initialize the ImToolsFeedback string
 **
 **  PROJECT
 **	IM		-  Image Manipulation Tools
 **
 **  DESCRIPTION
 **	imfeedback.c initializes the standard Image Tools feedback form
 **	global character string.
 **
 **  PUBLIC CONTENTS
 **			d =defined constant
 **			f =function
 **			m =defined macro
 **			t =typedef/struct/union
 **			v =variable
 **			? =other
 **
 **	ImToolsFeedback	v  Image Tools feedback form
 **
 **  PRIVATE CONTENTS
 **	none
 **
 **  HISTORY
 **	$Log:	imfeedback.raw.c,v $
 **	Revision 1.2  92/08/31  17:05:15  vle
 **	Updated copyright notice.
 **	
 **	Revision 1.1  91/10/03  13:19:49  nadeau
 **	Initial revision
 **	
 **/

#include "imtools.h"

public char *ImToolsFeedback = "\
\n\
This form has been automatically generated by use of the '-feedback' option\n\
on a tool written and distributed by the San Diego Supercomputer Center (SDSC).\n\
This form is provided so that users of SDSC software can send us bug reports,\n\
suggestions for enhancements, or general comments.\n\
\n\
After you have filled out the questionnaire that follows, please send this to:\n\
\n\
	E-mail:		info@sdsc.edu\n\
\n\
	Surface mail:	Software Products Information\n\
			San Diego Supercomputer Center\n\
			P.O. Box 85608\n\
			San Diego, CA  92186-9784\n\
\n\
	FAX:		(619) 534-5113\n\
\n\
Please include any accompanying data, source fragments and sample output.\n\
\n\
If you have questions about this form, or any of our software tools, please\n\
feel free to mail to any of the above or call us at:\n\
\n\
			(619) 534-5100  8am-5pm, Pacific time\n\
\n\
-------------------------------------------------------------------------------\n\
				User Feedback!\n\
\n\
Who are you?\n\
	Name:		\n\
	Organization:	\n\
	Group:		\n\
	Job Title:	\n\
	Phone number:	\n\
	FAX number:	\n\
	E-mail address:	\n\
	Surface mail address:\n\
\n\
What equipment are you using?\n\
	Machine Name/Model:	\n\
	OS Name/Version:	\n\
\n\
What is your bug or comment?  (please be verbose!)\n\
";
